@@ -1,6 +1,6 @@
 /*
 * ModSecurity for Apache 2.x, http://www.modsecurity.org/
-* Copyright (c) 2004-2011 Trustwave Holdings, Inc. (http://www.trustwave.com/)
+* Copyright (c) 2004-2013 Trustwave Holdings, Inc. (http://www.trustwave.com/)
 *
 * You may not use this file except in compliance with
 * the License.  You may obtain a copy of the License at
@@ -276,7 +276,8 @@ static apr_status_t modsecurity_tx_cleanup(void *data) {
 apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     const char *s = NULL;
     const apr_array_header_t *arr;
-    char *_cookies = NULL;
+    char *semicolon = NULL;
+    char *comma = NULL;
     apr_table_entry_t *te;
     int i;
 
@@ -390,11 +391,9 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     if (msr->matched_vars == NULL) return -1;
     apr_table_clear(msr->matched_vars);
 
-    if(msr->txcfg->max_rule_time > 0)   {
-        msr->perf_rules = apr_table_make(msr->mp, 8);
-        if (msr->perf_rules == NULL) return -1;
-        apr_table_clear(msr->perf_rules);
-    }
+    msr->perf_rules = apr_table_make(msr->mp, 8);
+    if (msr->perf_rules == NULL) return -1;
+    apr_table_clear(msr->perf_rules);
 
     /* Locate the cookie headers and parse them */
     arr = apr_table_elts(msr->request_headers);
@@ -402,12 +401,27 @@ apr_status_t modsecurity_tx_init(modsec_rec *msr) {
     for (i = 0; i < arr->nelts; i++) {
         if (strcasecmp(te[i].key, "Cookie") == 0) {
             if (msr->txcfg->cookie_format == COOKIES_V0) {
-                _cookies = apr_pstrdup(msr->mp, te[i].val);
-                while((*_cookies != 0)&&(*_cookies != ',')&&(*_cookies != ';')) _cookies++;
-                if(*_cookies == ',')
-                    parse_cookies_v0(msr, te[i].val, msr->request_cookies, ",");
-                else
+                semicolon = apr_pstrdup(msr->mp, te[i].val);
+                while((*semicolon != 0)&&(*semicolon != ';')) semicolon++;
+                if(*semicolon == ';')    {
                     parse_cookies_v0(msr, te[i].val, msr->request_cookies, ";");
+                } else  {
+                    comma = apr_pstrdup(msr->mp, te[i].val);
+                    while((*comma != 0)&&(*comma != ',')) comma++;
+                    if(*comma == ',')    {
+                        comma++;
+                        if(*comma == 0x20)   {// looks like comma is the separator
+                            if (msr->txcfg->debuglog_level >= 5) {
+                                msr_log(msr, 5, "Cookie v0 parser: Using comma as a separator. Semi-colon was not identified!");
+                            }
+                            parse_cookies_v0(msr, te[i].val, msr->request_cookies, ",");
+                        } else {
+                            parse_cookies_v0(msr, te[i].val, msr->request_cookies, ";");
+                        }
+                    } else  {
+                        parse_cookies_v0(msr, te[i].val, msr->request_cookies, ";");
+                    }
+                }
             } else {
                 parse_cookies_v1(msr, te[i].val, msr->request_cookies);
             }
